@@ -13,7 +13,6 @@
 
 #include <stdio.h>
 
-#if defined(AFS_SUN54_ENV)
 #define OSVERS "SunOS 5.6"
 #include <sys/types.h>
 #include <rx/xdr.h>
@@ -71,7 +70,7 @@ UnConvCmd(struct cmd_syndesc *as, void *arock)
 static int
 handleit(struct cmd_syndesc *as)
 {
-    register struct cmd_item *ti;
+    struct cmd_item *ti;
     char *dname;
     afs_int32 haspart = 0, hasDevice = 0;
     struct vfstab mnt;
@@ -186,7 +185,7 @@ handleit(struct cmd_syndesc *as)
 main(argc, argv)
      char **argv;
 {
-    register struct cmd_syndesc *ts;
+    struct cmd_syndesc *ts;
     afs_int32 code;
 
     if (geteuid() != 0) {
@@ -332,11 +331,7 @@ ProcessAfsInodes(devname, partition)
 		int p5;
 		quad *q;
 
-#if	defined(AFS_SUN56_ENV)
 		q = (quad *) & (p->di_ic.ic_lsize);
-#else
-		q = &(p->di_ic.ic_size);
-#endif
 
 		p1 = p->di_gen;
 		p2 = p->di_ic.ic_flags;
@@ -706,17 +701,3 @@ CheckMountedDevice(devName)
 	return 1;
     return 0;
 }
-
-#else /* !AFS_SUN54_ENV */
-
-#include "AFS_component_version_number.c"
-
-main(argc, argv)
-     int argc;
-     char *argv[];
-{
-    printf
-	("%s: **ONLY** supported for Solaris 2.4 and later ...\n",
-	 argv[0]);
-}
-#endif /* !AFS_SUN5_ENV */
