@@ -11,37 +11,27 @@
  *		     output of this program to be valid.
  */
 
-#include <sys/types.h>
-#ifndef _WIN32
-#include <sys/time.h>
-#include <sys/file.h>
-#else
-#include <fcntl.h>
-#include <io.h>
-#define L_SET SEEK_SET
-#endif
-#include <stdio.h>
-#include <ctype.h>
-#include <string.h>
-#include <errno.h>
-
 #include <afsconfig.h>
 #include <afs/param.h>
+
+#include <roken.h>
+
+#ifndef _WIN32
+#include <sys/file.h>
+#else
+#define L_SET SEEK_SET
+#endif
+#include <ctype.h>
+
 #include <afs/com_err.h>
-
-
 #include <afs/cmd.h>		/*Command line parsing */
 #include <afs/afsutil.h>
-#include <errno.h>
 #include <lock.h>
-#ifndef _WIN32
-#include <netinet/in.h>
-#endif
 #define UBIK_INTERNALS
 #include <ubik.h>
 #include <rx/xdr.h>
 #include <rx/rx.h>
-#include <afs/com_err.h>
+
 #include "ptint.h"
 #include "ptserver.h"
 #include "pterror.h"
@@ -113,8 +103,8 @@ int
 main(int argc, char **argv)
 {
 
-    register struct cmd_syndesc *cs;	/*Command line syntax descriptor */
-    register afs_int32 code;	/*Return code */
+    struct cmd_syndesc *cs;	/*Command line syntax descriptor */
+    afs_int32 code;	/*Return code */
 
     cs = cmd_CreateSyntax(NULL, CommandProc, NULL,
 			  "access protection database");
@@ -139,9 +129,9 @@ main(int argc, char **argv)
 }
 
 static int
-CommandProc(register struct cmd_syndesc *a_as, void *arock)
+CommandProc(struct cmd_syndesc *a_as, void *arock)
 {
-    register int i;
+    int i;
     long code = 0;
     long upos;
     long gpos = 0;
@@ -184,7 +174,7 @@ CommandProc(register struct cmd_syndesc *a_as, void *arock)
     }
 
     if (pfile == NULL) {
-        afs_snprintf(pbuffer, sizeof(pbuffer), "%s.DB0", pbase);
+        snprintf(pbuffer, sizeof(pbuffer), "%s.DB0", pbase);
         pfile = pbuffer;
     }
     if ((dbase_fd = open(pfile, (wflag ? O_RDWR : O_RDONLY) | O_CREAT, 0600))
@@ -342,7 +332,7 @@ CommandProc(register struct cmd_syndesc *a_as, void *arock)
 		long newpos;
 		newpos = display_entry(upos);
 		if (newpos == upos) {
-		    fprintf(stderr, "pt_util: hash error in %s chain %d\n", 
+		    fprintf(stderr, "pt_util: hash error in %s chain %d\n",
 			    nflag ? "name":"id", i);
 		    exit(1);
 		} else
@@ -401,7 +391,7 @@ static void
 add_group(long id)
 {
     struct grp_list *g;
-    register long i;
+    long i;
 
     i = grp_count++ % 1024;
     if (i == 0) {
@@ -416,7 +406,7 @@ add_group(long id)
 static void
 display_groups(void)
 {
-    register int i, id;
+    int i, id;
     struct grp_list *g;
 
     g = grp_head;
@@ -435,7 +425,7 @@ display_groups(void)
 static void
 display_group(int id)
 {
-    register int i, offset;
+    int i, offset;
     int print_grp = 0;
 
     offset = ntohl(prh.idHash[IDHash(id)]);
@@ -506,7 +496,7 @@ display_group(int id)
 static void
 fix_pre(struct prentry *pre)
 {
-    register int i;
+    int i;
 
     pre->flags = ntohl(pre->flags);
     pre->id = ntohl(pre->id);
@@ -533,7 +523,7 @@ fix_pre(struct prentry *pre)
 static char *
 id_to_name(int id)
 {
-    register int offset;
+    int offset;
     static struct prentry pre;
     char *name;
 
@@ -561,7 +551,7 @@ static char *
 checkin(struct prentry *pre)
 {
     struct hash_entry *he, *last;
-    register int id;
+    int id;
 
     id = pre->id;
     last = (struct hash_entry *)0;
@@ -588,7 +578,7 @@ checkin(struct prentry *pre)
 }
 
 static char *
-check_core(register int id)
+check_core(int id)
 {
     struct hash_entry *he;
     he = hat[IDHash(id)];
