@@ -18,12 +18,15 @@
 
 #include <linux/version.h>
 #include "rx/rx_kcommon.h"
-#if defined(HAVE_LINUX_UACCESS_H)
-#include <linux/uaccess.h>
-#else
+#include "rx.h"
+#include "rx_atomic.h"
+#include "rx_globals.h"
+#include "rx_stats.h"
+#include "rx_peer.h"
+#include "rx_packet.h"
+#include "rx_internal.h"
 #include <asm/uaccess.h>
-#endif
-#ifdef ADAPT_PMTU
+#ifdef AFS_RXERRQ_ENV
 #include <linux/errqueue.h>
 #include <linux/icmp.h>
 #endif
@@ -39,18 +42,16 @@ rxk_NewSocketHost(afs_uint32 ahost, short aport)
     struct socket *sockp;
     struct sockaddr_in myaddr;
     int code;
-#ifdef ADAPT_PMTU
+#ifdef AFS_ADAPT_PMTU
     int pmtu = IP_PMTUDISC_WANT;
-    int do_recverr = 1;
 #else
     int pmtu = IP_PMTUDISC_DONT;
 #endif
 
-#ifdef HAVE_LINUX_SOCK_CREATE_KERN_NS
-    code = sock_create_kern(&init_net, AF_INET, SOCK_DGRAM, IPPROTO_UDP, &sockp);
-#elif defined(HAVE_LINUX_SOCK_CREATE_KERN)
-    code = sock_create_kern(AF_INET, SOCK_DGRAM, IPPROTO_UDP, &sockp);
-#elif defined(LINUX_KERNEL_SOCK_CREATE_V)
+    /* We need a better test for this. if you need it back, tell us
+     * how to detect it. 
+     */
+#ifdef LINUX_KERNEL_SOCK_CREATE_V
     code = sock_create(AF_INET, SOCK_DGRAM, IPPROTO_UDP, &sockp, 0);
 #else
     code = sock_create(AF_INET, SOCK_DGRAM, IPPROTO_UDP, &sockp);
@@ -72,9 +73,12 @@ rxk_NewSocketHost(afs_uint32 ahost, short aport)
 
     kernel_setsockopt(sockp, SOL_IP, IP_MTU_DISCOVER, (char *)&pmtu,
 		      sizeof(pmtu));
-#ifdef ADAPT_PMTU
-    kernel_setsockopt(sockp, SOL_IP, IP_RECVERR, (char *)&do_recverr,
-                      sizeof(do_recverr));
+#ifdef AFS_RXERRQ_ENV
+    {
+	int recverr = 1;
+	kernel_setsockopt(sockp, SOL_IP, IP_RECVERR, (char *)&recverr,
+	                  sizeof(recverr));
+    }
 #endif
     return (osi_socket *)sockp;
 }
@@ -93,7 +97,7 @@ rxk_FreeSocket(struct socket *asocket)
     return 0;
 }
 
-#ifdef ADAPT_PMTU
+#ifdef AFS_RXERRQ_ENV
 void
 handle_socket_error(osi_socket so)
 {
@@ -135,13 +139,7 @@ handle_socket_error(osi_socket so)
 
     memcpy(&addr, offender, sizeof(addr));
 
-    if (err->ee_origin == SO_EE_ORIGIN_ICMP &&
-	err->ee_type == ICMP_DEST_UNREACH &&
-	err->ee_code == ICMP_FRAG_NEEDED) {
-	rxi_SetPeerMtu(NULL, ntohl(addr.sin_addr.s_addr), ntohs(addr.sin_port),
-		       err->ee_info);
-    }
-    /* other DEST_UNREACH's and TIME_EXCEEDED should be dealt with too */
+    rxi_ProcessNetError(err, addr.sin_addr.s_addr, addr.sin_port);
 
 out:
     rxi_Free(controlmsgbuf, 256);
@@ -161,7 +159,7 @@ osi_NetSend(osi_socket sop, struct sockaddr_in *to, struct iovec *iovec,
 {
     struct msghdr msg;
     int code;
-#ifdef ADAPT_PMTU
+#ifdef AFS_RXERRQ_ENV
     int sockerr;
     int esize;
 
@@ -213,7 +211,7 @@ osi_NetReceive(osi_socket so, struct sockaddr_in *from, struct iovec *iov,
 {
     struct msghdr msg;
     int code;
-#ifdef ADAPT_PMTU
+#ifdef AFS_RXERRQ_ENV
     int sockerr;
     int esize;
 #endif
@@ -223,7 +221,7 @@ osi_NetReceive(osi_socket so, struct sockaddr_in *from, struct iovec *iov,
     if (iovcnt > RX_MAXWVECS + 2) {
 	osi_Panic("Too many (%d) iovecs passed to osi_NetReceive\n", iovcnt);
     }
-#ifdef ADAPT_PMTU
+#ifdef AFS_RXERRQ_ENV
     while (1) {
 	sockerr=0;
 	esize = sizeof(sockerr);
@@ -235,13 +233,8 @@ osi_NetReceive(osi_socket so, struct sockaddr_in *from, struct iovec *iov,
 #endif
     memcpy(tmpvec, iov, iovcnt * sizeof(struct iovec));
     msg.msg_name = from;
-#if defined(STRUCT_MSGHDR_HAS_MSG_ITER)
-    msg.msg_iter.iov = tmpvec;
-    msg.msg_iter.nr_segs = iovcnt;
-#else
     msg.msg_iov = tmpvec;
     msg.msg_iovlen = iovcnt;
-#endif
     msg.msg_control = NULL;
     msg.msg_controllen = 0;
     msg.msg_flags = 0;
